#!/usr/bin/env bash

DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

source ${DIR}/../config/dms.conf
source ${DIR}/default/containers.conf
source ${DIR}/../config/containers.conf

CONFIG_FOLDER=${DIR}"/../config"

# Settings section
START_ALL=true
CHECK_GIT=true

# Log settings
LOG_LEVEL="ERROR"
LOG_EXTERNAL=false

# ------------ logic below ------------

RMC=true

if [ -n "$DMS_IMAGE_VERSION" ]
then
  NUXEO_IMAGE_VERSION=$DMS_IMAGE_VERSION
  CB_IMAGE_VERSION=$DMS_IMAGE_VERSION
  WORKER_IMAGE_VERSION=$DMS_IMAGE_VERSION
fi

if [ -z "$WORKER_IMAGE_VERSION" ]
then
  WORKER_IMAGE_VERSION=$NUXEO_IMAGE_VERSION
fi

for var in "$@"
do
  if [ ${var} = "-local" ]
  then
    CHECK_GIT=false
  elif [ ${var} = "-logsrv" ]
  then
    LOG_EXTERNAL=true
  elif [ ${var} = "-debug" ]
  then
    RUN_IN_DEBUG=true
  elif [ ${var} = "es" ]
  then
    START_ES=true
    START_ALL=false
  elif [ ${var} = "postgres" ]
  then
    START_POSTGRES=true
    START_ALL=false
  elif [ ${var} = "nuxeo" ]
  then
    START_NUXEO=true
    START_ALL=false
  elif [ ${var} = "cb" ]
  then
    START_CB=true
    START_ALL=false
  elif [ ${var} = "service" ]
  then
    START_SERVICE=true
    START_ALL=false
  elif [ ${var} = "postfix" ]
  then
    START_POSTFIX=true
    START_ALL=false
  elif [ ${var} = "postbox" ]
  then
    START_POSTBOX=true
    START_ALL=false
  elif [ ${var} = "nginx" ]
  then
    START_NGINX=true
    START_ALL=false
  elif [ ${var} = "office" ]
  then
     START_OFFICE=true
     START_ALL=false
  elif [ ${var} = "-vvv" ]
  then
    LOG_LEVEL="DEBUG"
  else
    echo "Unknown option ${var}"
  fi
done

if [[ ${START_ALL} = true ]] || ([[ ${HTTPS_ENABLED} = true ]] && ([[ ${START_NUXEO} = true ]] || [[ ${START_CB} = true ]]))
then
  START_ES=true
  START_POSTGRES=true
  START_NUXEO=true
  START_CB=true
  START_SERVICE=true
  START_NGINX=${HTTPS_ENABLED}
fi

function check_variable_set {
if [ -z "${!1}" ]; then
  echo "$1 required variable is not set"
  if [ -z "${2}" ]; then
    exit 1
  else
    eval "$1=$2"
    echo "Default value '$2' wil be used for variable $1"
  fi
fi
}

DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

SCRIPTS_FOLDER=${DIR}"/scripts"

source ${SCRIPTS_FOLDER}/update_origins.sh

if [[ ${CHECK_GIT} = true ]]
then
  LC_ALL=C git --work-tree=${CONFIG_FOLDER} --git-dir=${CONFIG_FOLDER}/.git  fetch
  OUTPUT="$(LC_ALL=C git --work-tree=${CONFIG_FOLDER} --git-dir=${CONFIG_FOLDER}/.git status)"

# git 1.10 has changed status strings so below if statements may need to be adapted (eg. "working tree clean" instead of "working directory clean")

  if [[ $OUTPUT == *"nothing to commit, working directory clean"* ]] || [[ $OUTPUT == *"nothing to commit, working tree clean"* ]]
  then
    echo "Configuration is in sync with client specific git repository"
  elif [[ $OUTPUT == *"Your branch is ahead"* ]]
  then
    ERROR_MESSAGE="Config folder has unpushed commits. Please rollback or push your changes."
  elif [[ $OUTPUT == *"Your branch is behind"* ]]
  then
    ERROR_MESSAGE="Config folder has unpulled commits. Please pull."
  elif [[ $OUTPUT == *"have diverged"* ]]
  then
    ERROR_MESSAGE="Config folder has unpulled and unpushed commits. Please resolve."
  else
  	ERROR_MESSAGE="Config folder is not in sync or cannot be verified with git repository, exiting."
  fi

  if [[ -n "${ERROR_MESSAGE}" ]]
  then
    echo ${ERROR_MESSAGE}
    exit 1
  fi
fi

source ${SCRIPTS_FOLDER}/create_network.sh

if [[ ${START_ES} = true ]]
then
  source ${SCRIPTS_FOLDER}/create_container.sh ES ${RMC}
fi

if [[ ${START_POSTGRES} = true ]]
then
  source ${SCRIPTS_FOLDER}/create_container.sh POSTGRES ${RMC}
fi

if [[ ${START_NUXEO} = true ]] || [[ ${START_CB} = true ]] || [[ ${START_POSTBOX} = true ]]
then
  source ${SCRIPTS_FOLDER}/create_conf_files.sh
  source ${SCRIPTS_FOLDER}/create_container.sh WORKER ${RMC}
fi

if [[ ${START_OFFICE} = true ]]
then
  source ${SCRIPTS_FOLDER}/create_container.sh OFFICE ${RMC}
  mkdir ${NUXEO_WINFONTDIR}/oo
  docker cp onlyoffice:/var/www/onlyoffice/documentserver/fonts ${NUXEO_WINFONTDIR}/oo
  docker cp onlyoffice:/var/www/onlyoffice/documentserver/core-fonts ${NUXEO_WINFONTDIR}/oo
  docker exec onlyoffice sed -i -- 's/"rejectUnauthorized": true/"rejectUnauthorized": false/g' /etc/onlyoffice/documentserver/default.json
  docker restart onlyoffice
fi

if [[ -z "${HTTPS_ENABLED}" ]] || [[ ${HTTPS_ENABLED} = false  ]]
then
  if [[ -n "${NGINX_CONTAINER_NAME}" ]]
  then
    docker stop ${NGINX_CONTAINER_NAME}
    docker rm ${NGINX_CONTAINER_NAME}
  fi
fi

if [[ ${START_NUXEO} = true ]]
then
source ${SCRIPTS_FOLDER}/create_container.sh NUXEO ${RMC} ${LOG_EXTERNAL}
 if [[ ${HTTPS_ENABLED} != true ]]
 then
   source ${SCRIPTS_FOLDER}/wait_for_container_start.sh 8080 NUXEO
 fi
fi

if [[ ${START_CB} = true ]]
then
source ${SCRIPTS_FOLDER}/create_container.sh CB ${RMC} ${LOG_EXTERNAL}
  if [[ ${HTTPS_ENABLED} != true ]]
  then
    source ${SCRIPTS_FOLDER}/wait_for_container_start.sh 9080 CB
  fi
fi

if [[ ${START_POSTBOX} = true ]]
then
  if [[ ${POSTBOX_ENABLED} != true ]]
  then
    echo "Postbox is not enabled in dms.conf - deployment will not continue!"
  else
    source ${SCRIPTS_FOLDER}/create_container.sh POSTBOX ${RMC}
    if [[ ${HTTPS_ENABLED} != true ]]
    then
      source ${SCRIPTS_FOLDER}/wait_for_container_start.sh 9083 POSTBOX
    else
      source ${SCRIPTS_FOLDER}/create_container.sh NGINX ${RMC}
      source ${SCRIPTS_FOLDER}/wait_for_https_container_start.sh POSTBOX
      docker cp "${DATADIR}/nginx/$HTTPS_CERTIFICATE" postbox:/nginx.crt
      docker exec -ti postbox keytool -import -noprompt -keypass "${HTTPS_CERTIFICATE_PASSWORD}" -storepass changeit -alias nuxeo  -keystore /etc/ssl/certs/java/cacerts -file /nginx.crt
    fi
  fi
fi

if [[ ${HTTPS_ENABLED} = true ]] && ([[ ${START_NUXEO} = true ]] || [[ ${START_CB} = true ]])
then
  source ${SCRIPTS_FOLDER}/create_container.sh NGINX ${RMC}

  source ${SCRIPTS_FOLDER}/wait_for_https_container_start.sh NUXEO
  docker cp "${DATADIR}/nginx/$HTTPS_CERTIFICATE" nuxeo:/nginx.crt
  docker exec -ti nuxeo keytool -import -noprompt -keypass "${HTTPS_CERTIFICATE_PASSWORD}" -storepass changeit -alias nuxeo  -keystore /etc/ssl/certs/java/cacerts -file /nginx.crt

  docker restart ${CB_CONTAINER_NAME}
  source ${SCRIPTS_FOLDER}/wait_for_https_container_start.sh CB
  docker cp "${DATADIR}/nginx/$HTTPS_CERTIFICATE" cb:/nginx.crt
  docker exec -ti cb keytool -import -noprompt -keypass "${HTTPS_CERTIFICATE_PASSWORD}" -storepass changeit -alias nuxeo  -keystore /etc/ssl/certs/java/cacerts -file /nginx.crt
fi

if [[ ${START_POSTFIX} = true ]]
then
  source ${SCRIPTS_FOLDER}/create_postfix_conf_files.sh
  source ${SCRIPTS_FOLDER}/create_container.sh POSTFIX ${RMC}
fi

if [[ ${START_SERVICE} = true ]]
then
  source ${SCRIPTS_FOLDER}/create_container.sh SERVICE ${RMC}
fi

source ${SCRIPTS_FOLDER}/send_health_report.sh

docker rmi `docker images -a -q`
